@@ -35,7 +35,7 @@ struct _XtmProcessStatusbarClass
 };
 struct _XtmProcessStatusbar
 {
-	GtkStatusbar		parent;
+	GtkHBox			parent;
 	/*<private>*/
 	XtmSettings *		settings;
 
@@ -45,11 +45,11 @@ struct _XtmProcessStatusbar
 	GtkWidget *		label_swap;
 
 	gfloat			cpu;
-	gfloat			memory;
-	gfloat			swap;
+	gchar			memory[64];
+	gchar			swap[64];
 	guint			num_processes;
 };
-G_DEFINE_TYPE (XtmProcessStatusbar, xtm_process_statusbar, GTK_TYPE_STATUSBAR)
+G_DEFINE_TYPE (XtmProcessStatusbar, xtm_process_statusbar, GTK_TYPE_BOX)
 
 static void	xtm_process_statusbar_finalize			(GObject *object);
 static void	xtm_process_statusbar_set_property		(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -66,9 +66,9 @@ xtm_process_statusbar_class_init (XtmProcessStatusbarClass *klass)
 	g_object_class_install_property (class, PROP_CPU,
 		g_param_spec_float ("cpu", "CPU", "CPU usage", 0, 100, 0, G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
 	g_object_class_install_property (class, PROP_MEMORY,
-		g_param_spec_float ("memory", "Memory", "Memory usage", 0, 100, 0, G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
+		g_param_spec_string ("memory", "Memory", "Memory usage", "", G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
 	g_object_class_install_property (class, PROP_SWAP,
-		g_param_spec_float ("swap", "Swap", "Swap usage", 0, 100, 0, G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
+		g_param_spec_string ("swap", "Swap", "Swap usage", "", G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
 	g_object_class_install_property (class, PROP_SHOW_SWAP,
 		g_param_spec_boolean ("show-swap", "ShowSwap", "Show or hide swap usage", TRUE, G_PARAM_WRITABLE));
 	g_object_class_install_property (class, PROP_NUM_PROCESSES,
@@ -78,42 +78,36 @@ xtm_process_statusbar_class_init (XtmProcessStatusbarClass *klass)
 static void
 xtm_process_statusbar_init (XtmProcessStatusbar *statusbar)
 {
-	GtkWidget *area, *hbox;
-
+	GtkWidget *hbox, *hbox_cpu, *hbox_mem;
 	statusbar->settings = xtm_settings_get_default ();
 
-#if GTK_CHECK_VERSION(2,20,0)
-	area = gtk_statusbar_get_message_area (GTK_STATUSBAR (statusbar));
+#if GTK_CHECK_VERSION(3, 0, 0)
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 16);
+	hbox_cpu = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 16);
+	hbox_mem = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 16);
 #else
-	{
-		GtkShadowType shadow_type;
-		GtkWidget *frame;
-
-		gtk_widget_style_get (GTK_WIDGET (statusbar), "shadow-type", &shadow_type, NULL);
-		frame = gtk_frame_new (NULL);
-		gtk_frame_set_shadow_type (GTK_FRAME (frame), shadow_type);
-		gtk_box_pack_start (GTK_BOX (statusbar), frame, TRUE, TRUE, 0);
-
-		area = gtk_hbox_new (FALSE, 0);
-		gtk_container_add (GTK_CONTAINER (frame), area);
-		gtk_widget_show_all (frame);
-	}
+	hbox = gtk_hbox_new (FALSE, 16);
+	hbox_cpu = gtk_hbox_new (FALSE, 16);
+	hbox_mem = gtk_hbox_new (FALSE, 16);
 #endif
 
-	hbox = gtk_hbox_new (FALSE, 16);
-	gtk_box_pack_start (GTK_BOX (area), hbox, TRUE, TRUE, 6);
+	statusbar->label_cpu = gtk_label_new (NULL);
+	gtk_box_pack_start (GTK_BOX (hbox_cpu), statusbar->label_cpu, TRUE, FALSE, 0);
 
 	statusbar->label_num_processes = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (hbox), statusbar->label_num_processes, FALSE, FALSE, 0);
-
-	statusbar->label_cpu = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (hbox), statusbar->label_cpu, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox_cpu), statusbar->label_num_processes, TRUE, FALSE, 0);
 
 	statusbar->label_memory = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (hbox), statusbar->label_memory, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox_mem), statusbar->label_memory, TRUE, FALSE, 0);
 
 	statusbar->label_swap = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (hbox), statusbar->label_swap, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox_mem), statusbar->label_swap, TRUE, FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_cpu, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), hbox_mem, TRUE, TRUE, 0);
+	gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
+
+	gtk_box_pack_start (GTK_BOX (statusbar), hbox, TRUE, TRUE, 0);
 
 	gtk_widget_show_all (hbox);
 }
@@ -139,6 +133,11 @@ xtm_process_statusbar_set_property (GObject *object, guint property_id, const GV
 	XtmProcessStatusbar *statusbar = XTM_PROCESS_STATUSBAR (object);
 	gchar *text;
 	gchar *float_value;
+#if GTK_CHECK_VERSION(3, 0, 0)
+	GdkRGBA color;
+#else
+	GdkColor color;
+#endif
 
 	switch (property_id)
 	{
@@ -147,25 +146,35 @@ xtm_process_statusbar_set_property (GObject *object, guint property_id, const GV
 		float_value = rounded_float_value (statusbar->cpu, statusbar->settings);
 		text = g_strdup_printf (_("CPU: %s%%"), float_value);
 		gtk_label_set_text (GTK_LABEL (statusbar->label_cpu), text);
+#if GTK_CHECK_VERSION(3, 0, 0)
+		gdk_rgba_parse (&color, "#ff6e00");
+		gtk_widget_override_color (statusbar->label_cpu, GTK_STATE_NORMAL, &color);
+#else
+		gdk_color_parse ("#ff6e00", &color);
+		gtk_widget_modify_fg (statusbar->label_cpu, GTK_STATE_NORMAL, &color);
+#endif
 		g_free (float_value);
 		g_free (text);
 		break;
 
 		case PROP_MEMORY:
-		statusbar->memory = g_value_get_float (value);
-		float_value = rounded_float_value (statusbar->memory, statusbar->settings);
-		text = g_strdup_printf (_("Memory: %s%%"), float_value);
+		g_strlcpy(statusbar->memory, g_value_get_string (value), 64);
+		text = g_strdup_printf (_("Memory: %s"), statusbar->memory);
 		gtk_label_set_text (GTK_LABEL (statusbar->label_memory), text);
-		g_free (float_value);
+#if GTK_CHECK_VERSION(3, 0, 0)
+		gdk_rgba_parse (&color, "#ab1852");
+		gtk_widget_override_color (statusbar->label_cpu, GTK_STATE_NORMAL, &color);
+#else
+		gdk_color_parse ("#ab1852", &color);
+		gtk_widget_modify_fg (statusbar->label_memory, GTK_STATE_NORMAL, &color);
+#endif
 		g_free (text);
 		break;
 
 		case PROP_SWAP:
-		statusbar->swap = g_value_get_float (value);
-		float_value = rounded_float_value (statusbar->swap, statusbar->settings);
-		text = g_strdup_printf (_("Swap: %s%%"), float_value);
+		g_strlcpy(statusbar->swap, g_value_get_string (value), 64);
+		text = g_strdup_printf (_("Swap: %s"), statusbar->swap);
 		gtk_label_set_text (GTK_LABEL (statusbar->label_swap), text);
-		g_free (float_value);
 		g_free (text);
 		break;
 
