@@ -15,7 +15,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-
+#include "process-tree-model.h"
 #include "process-tree-view.h"
 #include "task-manager.h"
 #include "settings.h"
@@ -48,6 +48,8 @@ struct _XtmProcessTreeView
 	/*<private>*/
 	GtkListStore *		model;
 	GtkTreeModel *		model_filter;
+	gchar *			cmd_filter;
+	GtkTreeModel *		model_tree;
 	GtkTreeViewColumn *	sort_column;
 	gushort			columns_positions[N_COLUMNS];
 	XtmSettings *		settings;
@@ -68,8 +70,7 @@ static void		column_clicked					(GtkTreeViewColumn *column, XtmProcessTreeView *
 static gboolean		visible_func					(GtkTreeModel *model, GtkTreeIter *iter, XtmProcessTreeView *treeview);
 static gboolean		search_func					(GtkTreeModel *model, gint column, const gchar *key, GtkTreeIter *iter, gpointer user_data);
 static void		settings_changed				(GObject *object, GParamSpec *pspec, XtmProcessTreeView *treeview);
-
-
+static void		expand_row					(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, XtmProcessTreeView *treeview);
 
 static void
 xtm_process_tree_view_class_init (XtmProcessTreeViewClass *klass)
@@ -85,6 +86,7 @@ xtm_process_tree_view_init (XtmProcessTreeView *treeview)
 	GtkCellRenderer *cell_text, *cell_right_aligned;
 	GtkTreeViewColumn *column;
 	gboolean visible;
+	gboolean tree;
 
 	treeview->settings = xtm_settings_get_default ();
 	g_signal_connect (treeview->settings, "notify", G_CALLBACK (settings_changed), treeview);
@@ -93,7 +95,7 @@ xtm_process_tree_view_init (XtmProcessTreeView *treeview)
 		gchar *uid_name;
 		get_owner_uid (&treeview->owner_uid, &uid_name);
 		g_free (uid_name);
-		g_object_get (treeview->settings, "show-all-processes", &treeview->show_all_processes_cached, NULL);
+		g_object_get (treeview->settings, "show-all-processes", &treeview->show_all_processes_cached, "process-tree", &tree, NULL);
 	}
 
 	/* Create tree view model */
@@ -108,7 +110,16 @@ xtm_process_tree_view_init (XtmProcessTreeView *treeview)
 	treeview->model_filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (treeview->model), NULL);
 	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (treeview->model_filter), (GtkTreeModelFilterVisibleFunc)visible_func, treeview, NULL);
 
-	g_object_set (treeview, "search-column", XTM_PTV_COLUMN_COMMAND, "model", treeview->model_filter, NULL);
+	treeview->model_tree = xtm_process_tree_model_new (GTK_TREE_MODEL (treeview->model_filter));
+
+	g_object_set (treeview, "search-column", XTM_PTV_COLUMN_COMMAND, "model", tree ? treeview->model_tree : treeview->model_filter, NULL);
+	if (tree)
+	{
+		gtk_tree_view_expand_all (GTK_TREE_VIEW (treeview));
+		g_signal_connect (treeview->model_tree, "row-has-child-toggled", G_CALLBACK (expand_row), treeview);
+	}
+
+	treeview->cmd_filter = NULL;
 
 	/* Create cell renderer for tree view columns */
 	cell_text = gtk_cell_renderer_text_new();
@@ -234,6 +245,12 @@ xtm_process_tree_view_finalize (GObject *object)
 		treeview->model_filter = NULL;
 	}
 
+	if (GTK_IS_TREE_MODEL (treeview->model_tree))
+	{
+		g_object_unref (treeview->model_tree);
+		treeview->model_tree = NULL;
+	}
+
 	if (XTM_IS_SETTINGS (treeview->settings))
 	{
 		g_object_unref (treeview->settings);
@@ -311,7 +328,7 @@ read_columns_positions (XtmProcessTreeView *treeview)
 	else
 	{
 		columns_positions_split = g_strsplit (columns_positions, ";", N_COLUMNS + 1);
-		for (i = 0; columns_positions_split[i] != NULL && i < N_COLUMNS; i++)
+		for (i = 0; i < N_COLUMNS && columns_positions_split[i] != NULL; i++)
 			treeview->columns_positions[i] = (gint)g_ascii_strtoll (columns_positions_split[i], NULL, 10);
 		g_strfreev (columns_positions_split);
 		g_free (columns_positions);
@@ -409,12 +426,6 @@ build_context_menu (XtmProcessTreeView *treeview, guint pid)
 
 	menu = gtk_menu_new ();
 
-	mi = gtk_menu_item_new_with_label (_("Terminate"));
-	g_object_set_data (G_OBJECT (mi), "pid", GUINT_TO_POINTER (pid));
-	g_object_set_data (G_OBJECT (mi), "treeview", treeview);
-	gtk_container_add (GTK_CONTAINER (menu), mi);
-	g_signal_connect (mi, "activate", G_CALLBACK (cb_send_signal), GINT_TO_POINTER (XTM_SIGNAL_TERMINATE));
-
 	if (!pid_is_sleeping (pid))
 	{
 		mi = gtk_menu_item_new_with_label (_("Stop"));
@@ -430,6 +441,12 @@ build_context_menu (XtmProcessTreeView *treeview, guint pid)
 		g_signal_connect (mi, "activate", G_CALLBACK (cb_send_signal), GINT_TO_POINTER (XTM_SIGNAL_CONTINUE));
 	}
 
+	mi = gtk_menu_item_new_with_label (_("Terminate"));
+	g_object_set_data (G_OBJECT (mi), "pid", GUINT_TO_POINTER (pid));
+	g_object_set_data (G_OBJECT (mi), "treeview", treeview);
+	gtk_container_add (GTK_CONTAINER (menu), mi);
+	g_signal_connect (mi, "activate", G_CALLBACK (cb_send_signal), GINT_TO_POINTER (XTM_SIGNAL_TERMINATE));
+
 	mi = gtk_menu_item_new_with_label (_("Kill"));
 	g_object_set_data (G_OBJECT (mi), "pid", GUINT_TO_POINTER (pid));
 	gtk_container_add (GTK_CONTAINER (menu), mi);
@@ -509,8 +526,10 @@ treeview_clicked (XtmProcessTreeView *treeview, GdkEventButton *event)
 		GtkTreeSelection *selection;
 		GtkTreePath *path;
 		GtkTreeIter iter;
+		gboolean tree;
 
-		model = GTK_TREE_MODEL (treeview->model_filter);
+		g_object_get (treeview->settings, "process-tree", &tree, NULL);
+		model = GTK_TREE_MODEL (tree ? treeview->model_tree : treeview->model_filter);
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview), (gint)event->x, (gint)event->y, &path, NULL, NULL, NULL);
 
@@ -537,7 +556,7 @@ treeview_key_pressed (XtmProcessTreeView *treeview, GdkEventKey *event)
 {
 	guint pid;
 
-	if (event->keyval != GDK_Menu)
+	if (event->keyval != GDK_KEY_Menu)
 		return FALSE;
 
 	{
@@ -590,16 +609,45 @@ column_clicked (GtkTreeViewColumn *column, XtmProcessTreeView *treeview)
 	treeview->sort_column = column;
 }
 
+void
+xtm_process_tree_view_set_filter(XtmProcessTreeView *treeview, const gchar *cmd_filter)
+{
+	g_free(treeview->cmd_filter);
+	treeview->cmd_filter = g_strdup(cmd_filter);
+
+	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (treeview->model_filter));
+}
+
 static gboolean
 visible_func (GtkTreeModel *model, GtkTreeIter *iter, XtmProcessTreeView *treeview)
 {
+	gchar *cmdline, *cmdline_lower, *key_lower, *p = NULL;
+	gboolean  mach_filter = TRUE, match_uid = TRUE;
 	guint uid;
 
-	if (treeview->show_all_processes_cached)
-		return TRUE;
+	if(treeview->cmd_filter) {
+		gtk_tree_model_get (GTK_TREE_MODEL (model), iter, XTM_PTV_COLUMN_COMMAND, &cmdline, -1);
+		if(cmdline) {
+			cmdline_lower = g_ascii_strdown (cmdline, -1);
+			key_lower = g_ascii_strdown (treeview->cmd_filter, -1);
 
-	gtk_tree_model_get (GTK_TREE_MODEL (treeview->model), iter, XTM_PTV_COLUMN_UID, &uid, -1);
-	return (treeview->owner_uid == uid) ? TRUE : FALSE;
+			p = g_strrstr (cmdline_lower, key_lower);
+
+			g_free (key_lower);
+			g_free (cmdline_lower);
+			g_free (cmdline);
+		}
+
+		if(!p)
+			mach_filter = FALSE;
+	}
+	if (!treeview->show_all_processes_cached) {
+		gtk_tree_model_get (GTK_TREE_MODEL (treeview->model), iter, XTM_PTV_COLUMN_UID, &uid, -1);
+		if (treeview->owner_uid != uid)
+			match_uid = FALSE;
+	}
+
+	return (mach_filter && match_uid);
 }
 
 static gboolean
@@ -661,6 +709,28 @@ settings_changed (GObject *object, GParamSpec *pspec, XtmProcessTreeView *treevi
 		g_object_get (object, pspec->name, &treeview->show_all_processes_cached, NULL);
 		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (treeview->model_filter));
 	}
+	else if (!g_strcmp0 (pspec->name, "process-tree"))
+	{
+		gboolean tree;
+		g_object_get (object, pspec->name, &tree, NULL);
+		gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), tree ? treeview->model_tree : treeview->model_filter);
+		if (tree)
+		{
+			gtk_tree_view_expand_all (GTK_TREE_VIEW (treeview));
+			g_signal_connect (treeview->model_tree, "row-has-child-toggled", G_CALLBACK (expand_row), treeview);
+		}
+		else
+		{
+			g_signal_handlers_disconnect_by_func (treeview->model_tree, G_CALLBACK (expand_row), treeview);
+		}
+	}
+}
+
+
+static void
+expand_row (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, XtmProcessTreeView *treeview)
+{
+	gtk_tree_view_expand_row (GTK_TREE_VIEW (treeview), path, FALSE);
 }
 
 
@@ -678,3 +748,9 @@ xtm_process_tree_view_get_sort_column_id (XtmProcessTreeView *treeview, gint *so
 	*sort_type = gtk_tree_view_column_get_sort_order (treeview->sort_column);
 }
 
+GtkTreeModel *
+xtm_process_tree_view_get_model (XtmProcessTreeView *treeview)
+{
+	return GTK_TREE_MODEL (treeview->model);
+}
+
