@@ -52,5 +52,7 @@ typedef struct _XtmProcessTreeView XtmProcessTreeView;
 GType		xtm_process_tree_view_get_type			(void);
 GtkWidget *	xtm_process_tree_view_new			(void);
 void		xtm_process_tree_view_get_sort_column_id	(XtmProcessTreeView *treeview, gint *sort_column_id, GtkSortType *sort_type);
+void		xtm_process_tree_view_set_filter		(XtmProcessTreeView *treeview, const gchar *cmd_filter);
+GtkTreeModel *	xtm_process_tree_view_get_model			(XtmProcessTreeView *treeview);
 
 #endif /* !PROCESS_TREE_VIEW_H */
