@@ -36,6 +36,7 @@ enum
 	PROP_MORE_PRECISION,
 	PROP_FULL_COMMAND_LINE,
 	PROP_SHOW_STATUS_ICON,
+	PROP_SHOW_MEMORY_IN_XBYTES,
 	PROP_MONITOR_PAINT_BOX,
 	PROP_SHOW_APPLICATION_ICONS,
 	PROP_TOOLBAR_STYLE,
@@ -54,6 +55,8 @@ enum
 	PROP_SORT_TYPE,
 	PROP_WINDOW_WIDTH,
 	PROP_WINDOW_HEIGHT,
+	PROP_HANDLE_POSITION,
+	PROP_PROCESS_TREE,
 	N_PROPS,
 };
 typedef struct _XtmSettingsClass XtmSettingsClass;
@@ -92,7 +95,9 @@ xtm_settings_class_init (XtmSettingsClass *klass)
 	g_object_class_install_property (class, PROP_FULL_COMMAND_LINE,
 		g_param_spec_boolean ("full-command-line", "FullCommandLine", "Full command line", FALSE, G_PARAM_READWRITE));
 	g_object_class_install_property (class, PROP_SHOW_STATUS_ICON,
-		g_param_spec_boolean ("show-status-icon", "ShowStatusIcon", "Show/hide the status icon", TRUE, G_PARAM_READWRITE));
+		g_param_spec_boolean ("show-status-icon", "ShowStatusIcon", "Show/hide the status icon", FALSE, G_PARAM_READWRITE));
+	g_object_class_install_property (class, PROP_SHOW_MEMORY_IN_XBYTES,
+		g_param_spec_boolean ("show-memory-in-xbytes", "ShowMemoryInXBytes", "Show memory usage in bytes", FALSE, G_PARAM_READWRITE));
 	g_object_class_install_property (class, PROP_MONITOR_PAINT_BOX,
 		g_param_spec_boolean ("monitor-paint-box", "MonitorPaintBox", "Paint box around monitor", TRUE, G_PARAM_READWRITE));
 	g_object_class_install_property (class, PROP_SHOW_APPLICATION_ICONS,
@@ -129,6 +134,10 @@ xtm_settings_class_init (XtmSettingsClass *klass)
 		g_param_spec_int ("window-width", "WindowWidth", "Window width", -1, G_MAXINT, -1, G_PARAM_READWRITE));
 	g_object_class_install_property (class, PROP_WINDOW_HEIGHT,
 		g_param_spec_int ("window-height", "WindowHeight", "Window height", -1, G_MAXINT, -1, G_PARAM_READWRITE));
+	g_object_class_install_property (class, PROP_HANDLE_POSITION,
+		g_param_spec_int ("handle-position", "HandlePosition", "Handle position", -1, G_MAXINT, -1, G_PARAM_READWRITE));
+	g_object_class_install_property (class, PROP_PROCESS_TREE,
+		g_param_spec_boolean ("process-tree", "ProcessTreeView", "Process tree", FALSE, G_PARAM_READWRITE));
 }
 
 static void
