@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2010 Landry Breuil <landry@xfce.org>
+ * Copyright (c) 2008-2014 Landry Breuil <landry@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,8 +31,15 @@
 #include <string.h>
 /* for getpagesize() */
 #include <unistd.h>
+/* for P_ZOMBIE & SSLEEP */
+#include <sys/proc.h>
+/* for struct vmtotal */
+#include <sys/vmmeter.h>
 #include "task-manager.h"
 
+#include <errno.h>
+extern int errno;
+
 char	*state_abbrev[] = {
 	"", "start", "run", "sleep", "stop", "zomb", "dead", "onproc"
 };
@@ -48,8 +55,8 @@ gboolean get_task_list (GArray *task_list)
 #endif
 	Task t;
 	struct passwd *passwdp;
-	char **args, **ptr;
-	char buf[127];
+	char **args;
+	gchar* buf;
 	int nproc, i;
 
 	mib[0] = CTL_KERN;
@@ -108,25 +115,28 @@ gboolean get_task_list (GArray *task_list)
 		if (!P_ZOMBIE(&p)) {
 			size = 128;
 			if ((args = malloc(size)) == NULL)
-				errx(1,"failed to allocate memory for argv structures");
+				errx(1,"failed to allocate memory for argv structures at %zu", size);
 			for (;; size *= 2) {
 				if ((args = realloc(args, size)) == NULL)
-					errx(1,"failed to allocate memory for argv structures of pid %d",t.pid);
+					errx(1,"failed to allocate memory (size=%zu) for argv structures of pid %d", size, t.pid);
 				mib[0] = CTL_KERN;
 				mib[1] = KERN_PROC_ARGS;
 				mib[2] = t.pid;
 				mib[3] = KERN_PROC_ARGV;
 				if (sysctl(mib, 4, args, &size, NULL, 0) == 0)
 					break;
+				if (errno != ENOMEM) { /* ESRCH: process disappeared */
+					 /* printf ("process with pid %d disappeared, errno=%d\n", t.pid, errno); */
+					args[0] ='\0';
+					args[1] = NULL;
+					break;
+				}
 			}
-			buf[0] = '\0';
-			for (ptr = args; *ptr != NULL; ptr++) {
-				if (ptr != args)
-					strlcat(buf, " ", sizeof(buf));
-				strlcat(buf, *ptr, sizeof(buf));
-			}
+			buf = g_strjoinv(" ", args);
+			g_assert(g_utf8_validate(buf, -1, NULL));
+			g_strlcpy(t.cmdline, buf, sizeof t.cmdline);
+			g_free(buf);
 			free(args);
-			g_snprintf(t.cmdline, sizeof t.cmdline, "%s", buf);
 		}
 
 		t.cpu_user = (100.0 * ((double) p.p_pctcpu / FSCALE));
@@ -174,7 +184,7 @@ pid_is_sleeping (guint pid)
 #else
 		errx(1, "could not read kern.proc2 for pid %d", pid);
 #endif
-	return (kp.p_stat == SSLEEP ? TRUE : FALSE);
+	return (kp.p_stat == SSTOP ? TRUE : FALSE);
 }
 
 gboolean get_cpu_usage (gushort *cpu_count, gfloat *cpu_user, gfloat *cpu_system)
@@ -213,11 +223,26 @@ gboolean get_cpu_usage (gushort *cpu_count, gfloat *cpu_user, gfloat *cpu_system
 
 gboolean get_memory_usage (guint64 *memory_total, guint64 *memory_free, guint64 *memory_cache, guint64 *memory_buffers, guint64 *swap_total, guint64 *swap_free)
 {
+#ifdef __OpenBSD__
+	int mib[] = {CTL_VM, VM_UVMEXP};
+	struct uvmexp  uvmexp;
+#else
 	int mib[] = {CTL_VM, VM_METER};
 	struct vmtotal vmtotal;
+#endif
 	struct swapent *swdev;
 	int nswap, i;
 	size_t size;
+#ifdef __OpenBSD__
+	size = sizeof(uvmexp);
+	if (sysctl(mib, 2, &uvmexp, &size, NULL, 0) < 0)
+		errx(1,"failed to get vm.uvmexp");
+	/* cheat : rm = tot used, add free to get total */
+	*memory_free = pagetok((guint64)uvmexp.free);
+	*memory_total = pagetok((guint64)uvmexp.npages);
+	*memory_cache = 0;
+	*memory_buffers = 0; /*pagetok(uvmexp.npages - uvmexp.free - uvmexp.active);*/
+#else
 	size = sizeof(vmtotal);
 	if (sysctl(mib, 2, &vmtotal, &size, NULL, 0) < 0)
 		errx(1,"failed to get vm.meter");
@@ -226,6 +251,7 @@ gboolean get_memory_usage (guint64 *memory_total, guint64 *memory_free, guint64
 	*memory_free = pagetok(vmtotal.t_free);
 	*memory_cache = 0;
 	*memory_buffers = pagetok(vmtotal.t_rm - vmtotal.t_arm);
+#endif
 
 	/* get swap stats */
 	if ((nswap = swapctl(SWAP_NSWAP, 0, 0)) == 0)
@@ -243,10 +269,12 @@ gboolean get_memory_usage (guint64 *memory_total, guint64 *memory_free, guint64
 	*swap_total = *swap_free = 0;
 	for (i = 0; i < nswap; i++) {
 		if (swdev[i].se_flags & SWF_ENABLE) {
-			*swap_free += ((swdev[i].se_nblks - swdev[i].se_inuse) / DEV_BSIZE);
-			*swap_total += (swdev[i].se_nblks / DEV_BSIZE);
+			*swap_free += (swdev[i].se_nblks - swdev[i].se_inuse);
+			*swap_total += swdev[i].se_nblks;
 		}
 	}
+	*swap_total *= DEV_BSIZE;
+	*swap_free *= DEV_BSIZE;
 	free(swdev);
 	return TRUE;
 }
