@@ -138,7 +138,7 @@ static gchar *
 pretty_cmdline (gchar *cmdline, gchar *comm)
 {
 	gchar *text = g_strchomp (g_strdelimit (g_strdup (cmdline), "\n\r", ' '));
-	if (!full_cmdline && sizeof (text) > 3)
+	if (!full_cmdline && g_utf8_strlen (text, -1) > 3)
 	{
 		/* Shorten full path to commands and wine applications */
 		if (text[0] == '/' || (g_ascii_isupper (text[0]) && text[1] == ':' && text[2] == '\\'))
@@ -146,7 +146,7 @@ pretty_cmdline (gchar *cmdline, gchar *comm)
 			gchar *p = g_strstr_len (text, -1, comm);
 			if (p != NULL)
 			{
-				g_snprintf (text, g_utf8_strlen (text, -1), "%s", p);
+				g_strlcpy (text, p, g_utf8_strlen (text, -1));
 			}
 		}
 	}
@@ -206,8 +206,8 @@ model_mark_tree_iter_as_removed (GtkTreeModel *model, GtkTreeIter *iter)
 	gtk_list_store_set (GTK_LIST_STORE (model), iter,
 		XTM_PTV_COLUMN_CPU, 0.0,
 		XTM_PTV_COLUMN_CPU_STR, "-",
-		XTM_PTV_COLUMN_BACKGROUND, "#a40000",
-		XTM_PTV_COLUMN_FOREGROUND, "#ffffff",
+		XTM_PTV_COLUMN_BACKGROUND, "#e57373",
+		XTM_PTV_COLUMN_FOREGROUND, "#000000",
 		XTM_PTV_COLUMN_TIMESTAMP, __current_timestamp (),
 		-1);
 }
@@ -218,28 +218,6 @@ model_remove_tree_iter (GtkTreeModel *model, GtkTreeIter *iter)
 	gtk_list_store_remove (GTK_LIST_STORE (model), iter);
 }
 
-static void
-memory_human_size (guint64 mem, gchar *mem_str)
-{
-	guint64 mem_tmp;
-
-	mem_tmp = mem / 1024 / 1024;
-	if (mem_tmp > 3)
-	{
-		g_snprintf (mem_str, 64, _("%lu MiB"), (gulong)mem_tmp);
-		return;
-	}
-
-	mem_tmp = mem / 1024;
-	if (mem_tmp > 8)
-	{
-		g_snprintf (mem_str, 64, _("%lu KiB"), (gulong)mem_tmp);
-		return;
-	}
-
-	g_snprintf (mem_str, 64, _("%lu B"), (gulong)mem);
-}
-
 static void
 #ifdef HAVE_WNCK
 model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task, App *app)
@@ -247,7 +225,7 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task, App
 model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 #endif
 {
-	gchar vsz[64], rss[64], cpu[16];
+	gchar *vsz, *rss, cpu[16];
 	gchar value[14];
 	glong old_timestamp;
 	gchar *old_state;
@@ -256,8 +234,8 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 	GdkPixbuf *icon;
 #endif
 
-	memory_human_size (task->vsz, vsz);
-	memory_human_size (task->rss, rss);
+	vsz = g_format_size_full (task->vsz, G_FORMAT_SIZE_IEC_UNITS);
+	rss = g_format_size_full (task->rss, G_FORMAT_SIZE_IEC_UNITS);
 
 	g_snprintf (value, 14, (more_precision) ? "%.2f" : "%.0f", task->cpu_user + task->cpu_system);
 	g_snprintf (cpu, 16, _("%s%%"), value);
@@ -293,7 +271,7 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 	if (g_strcmp0 (task->state, old_state) != 0 && background == NULL)
 	{
 		/* Set yellow color for changing state */
-		background = g_strdup ("#edd400");
+		background = g_strdup ("#fff176");
 		foreground = g_strdup ("#000000");
 		old_timestamp = __current_timestamp () - TIMESTAMP_DELTA + 3;
 	}
@@ -301,7 +279,7 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 	if (__current_timestamp () - old_timestamp <= TIMESTAMP_DELTA)
 	{
 		/* Set green color for started task */
-		background = (background == NULL) ? g_strdup ("#73d216") : background;
+		background = (background == NULL) ? g_strdup ("#aed581") : background;
 		foreground = (foreground == NULL) ? g_strdup ("#000000") : foreground;
 	}
 	else
@@ -329,6 +307,8 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 	g_free (background);
 	g_free (foreground);
 	g_free (old_state);
+	g_free (vsz);
+	g_free (rss);
 }
 
 static void
@@ -396,10 +376,11 @@ xtm_task_manager_get_hostname (XtmTaskManager *manager)
 }
 
 void
-xtm_task_manager_get_system_info (XtmTaskManager *manager, guint *num_processes, gfloat *cpu, gfloat *memory, gfloat *swap)
-{
-	guint64 memory_used, swap_used;
+xtm_task_manager_get_system_info (XtmTaskManager *manager, guint *num_processes, gfloat *cpu,
+				  guint64 *memory_used, guint64 *memory_total,
+				  guint64 *swap_used, guint64 *swap_total)
 
+{
 	g_return_if_fail (XTM_IS_TASK_MANAGER (manager));
 
 	/* Set number of processes */
@@ -409,11 +390,10 @@ xtm_task_manager_get_system_info (XtmTaskManager *manager, guint *num_processes,
 	get_memory_usage (&manager->memory_total, &manager->memory_free, &manager->memory_cache, &manager->memory_buffers,
 			&manager->swap_total, &manager->swap_free);
 
-	memory_used = manager->memory_total - manager->memory_free - manager->memory_cache - manager->memory_buffers;
-	swap_used = manager->swap_total - manager->swap_free;
-
-	*memory = (manager->memory_total != 0) ? memory_used * 100 / (gdouble)manager->memory_total : 0;
-	*swap = (manager->swap_total != 0) ? swap_used * 100 / (gdouble)manager->swap_total : 0;
+	*memory_used = manager->memory_total - manager->memory_free - manager->memory_cache - manager->memory_buffers;
+	*memory_total = manager->memory_total;
+	*swap_used = manager->swap_total - manager->swap_free;
+	*swap_total = manager->swap_total;
 
 	/* Set CPU usage */
 	get_cpu_usage (&manager->cpu_count, &manager->cpu_user, &manager->cpu_system);
